#include "delay.h"
#include "uart.h"
#include "stdio.h"
#include "pwm_generator.h"

extern void SysClockConfig(void);

/*Variable to hold the measured pulse length*/
uint32_t period;

/*Variable to hold the measured duty cycle*/
uint32_t duty_cycle;


/*variable to hold the measured distance*/
uint32_t frequency, dutyCycle;

int main(void)
{



	/*Enable float point hardware*/
	SCB->CPACR |= ((3UL << 10*2)|(3UL << 11*2));
	/*Set the clock to 100MHz*/
	SysClockConfig();

	/*Initialize UART2*/
	uart2_init();

	/*Initialize TIM1 as PWM*/
	TIM1_PWM_Init(0,600,300);

	/*Enable clock access to GPIOA*/
	RCC->AHB1ENR|=RCC_AHB1ENR_GPIOAEN;

	/*Set PA0 and PA1 to alternate function*/
	GPIOA->MODER|=(GPIO_MODER_MODE0_1);
	GPIOA->MODER&=~(GPIO_MODER_MODE0_0);

	/*Select AF01 for TIM2*/
	#define AF01 0x01
	GPIOA->AFR[0]|=(AF01<<GPIO_AFRL_AFSEL0_Pos)|(AF01<<GPIO_AFRL_AFSEL1_Pos);

	/*enable clock access to tim2*/
	RCC->APB1ENR|=RCC_APB1ENR_TIM2EN;

	/*set the prescaler to 99 to get 1MHZ*/
	TIM2->PSC=0;

	/*set the maximum to 199999 to get 5 Hz update rate*/
	TIM2->ARR=0xFFFFFFFF;

	/*Reset current counter*/
	TIM2->CNT=0;

	/*Set TIM2_Ch1 to be input mapped to TI1*/
	TIM2->CCMR1|=TIM_CCMR1_CC1S_0;

	/*Select TIM2 CH1 to be Rising edge*/
	TIM2->CCER&=~(TIM_CCER_CC1P|TIM_CCER_CC1NP);

	/*Set TIM2_CH2 to be input mapped to TI1*/
	TIM2->CCMR1|=TIM_CCMR1_CC2S_1;

	/*Select TIM2 CH1 to be Falling edge*/
	TIM2->CCER|=TIM_CCER_CC2P;
	TIM2->CCER&=~TIM_CCER_CC2NP;

	/*Set trigger mode to be Filtered Timer Input 1*/
	TIM2->SMCR|=TIM_SMCR_TS_2|TIM_SMCR_TS_0;

	/*Set the slave mode to be reset mode*/
	TIM2->SMCR|=TIM_SMCR_SMS_2;

	/*Enable both channel*/
	TIM2->CCER|=TIM_CCER_CC1E|TIM_CCER_CC2E;

	/*Enable DMA for both channel*/

	TIM2->DIER|=TIM_DIER_CC1DE|TIM_DIER_CC2DE;


	/*DMA configuration*/

	/*Enable clock access to DMA1*/

	RCC->AHB1ENR|=RCC_AHB1ENR_DMA1EN;

	/*Disable the stream and wait until it is disabled*/
	DMA1_Stream6->CR &=~DMA_SxCR_EN;

	while((DMA1_Stream6->CR & DMA_SxCR_EN) == DMA_SxCR_EN );

	/*Configure the DMA with the following parameters
	 *
	 * Channel is channel 3
	 * Memory and peripheral size is word (32-bit)
	 * Circular mode
	 *
	 * */


	#define CH3 0x03

	DMA1_Stream6->CR |= (CH3<<DMA_SxCR_CHSEL_Pos)|DMA_SxCR_PSIZE_1|DMA_SxCR_MSIZE_1|DMA_SxCR_CIRC;

	/*Set number of transfers to be 1
	 * Peripheral address to TIM2->CCR2
	 * Memory address to be the period variable
	 * */
	DMA1_Stream6->NDTR=1;
	DMA1_Stream6->PAR= (uint32_t)&TIM2->CCR2;
	DMA1_Stream6->M0AR=(uint32_t)&duty_cycle;

	/*Enable DMA*/
	DMA1_Stream6->CR |=DMA_SxCR_EN;



	/*Disable the stream and wait until it is disabled*/
	DMA1_Stream5->CR &=~DMA_SxCR_EN;

	while((DMA1_Stream5->CR & DMA_SxCR_EN) == DMA_SxCR_EN );

	/*Configure the DMA with the following parameters
	 *
	 * Channel is channel 3
	 * Memory and peripheral size is word (32-bit)
	 * Circular mode
	 *
	 * */

	DMA1_Stream5->CR |= (CH3<<DMA_SxCR_CHSEL_Pos)|DMA_SxCR_PSIZE_1|DMA_SxCR_MSIZE_1|DMA_SxCR_CIRC;

	/*Set number of transfers to be 1
	 * Peripheral address to TIM2->CCR1
	 * Memory address to be the period variable
	 * */
	DMA1_Stream5->NDTR=1;
	DMA1_Stream5->PAR= (uint32_t)&TIM2->CCR1;
	DMA1_Stream5->M0AR=(uint32_t)&period;

	/*Enable DMA*/
	DMA1_Stream5->CR |=DMA_SxCR_EN;

	/*Enable timer*/
	TIM2->CR1=TIM_CR1_CEN;



	while(1)
	{

		frequency=(float)(100000000/(period+1));

		dutyCycle=(float)(((duty_cycle+1)*100)/(period));

		printf("Period is %ld\r\n",frequency);

		printf("Duty cycle is %ld\r\n",dutyCycle);
		delay(200);


	}

}


